package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.MappingConfig;
import com.diagbot.entity.wrapper.MappingConfigWrapper;
import com.diagbot.facade.MappingConfigFacade;
import com.diagbot.util.BeanUtil;
import com.diagbot.vo.IdListVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.MappingConfigPageVO;
import com.diagbot.vo.MappingConfigVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/6/10 16:34
 */
@RestController
@RequestMapping("/tran/mappingConfig")
@Api(value = "术语关联相关API", tags = { "术语关联相关API" })
@SuppressWarnings("unchecked")
public class MappingConfigController {
    @Autowired
    private MappingConfigFacade mappingConfigFacade;


    @ApiOperation(value = "数据导入模板导出[by:zhaops]",
            notes = "type（必填）： 类型：1-化验、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候、14-麻醉 <br>")
    @PostMapping("/exportExcelModule")
    @SysLogger("exportExcelModule")
    public void exportExcelModule(HttpServletResponse response, @RequestBody @Valid MappingConfigVO mappingConfigVO) {
        mappingConfigFacade.exportExcelModule(response, mappingConfigVO);
    }

    @ApiOperation(value = "预匹配导入术语校验[by:zhaops]",
            notes = "file（必填）：预匹配文件 <br>" +
                    "type（必填）： 类型：1-化验、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候、14-麻醉 <br>")
    @PostMapping(value = "/dataVerify", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @SysLogger("dataVerify")
    public RespDTO<Boolean> dataVerify(@RequestParam("file") MultipartFile file,
                                       @RequestParam("type") Integer type) {
        Boolean data = mappingConfigFacade.dataVerify(file, type);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "预匹配[by:zhaops]",
            notes = "file（必填）：预匹配文件 <br>" +
                    "type（必填）： 类型：1-化验、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候、14-麻醉 <br>")
    @PostMapping(value = "/precDataMatch", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @SysLogger("precDataMatch")
    public void precDataMatch(@RequestParam("file") MultipartFile file,
                              HttpServletResponse response,
                              @RequestParam("type") Integer type) {
        mappingConfigFacade.precDataMatch(file, type, response);
    }

    @ApiOperation(value = "预匹配_远程调用[by:zhaops]", notes = "")
    @PostMapping(value = "/precDataMatch_remote", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @SysLogger("precDataMatch_remote")
    @ApiIgnore
    public RespDTO<List<MappingConfigWrapper>> precDataMatch_remote(@RequestParam("file") MultipartFile file,
                                                                    @RequestParam("type") Integer type) {
        List<MappingConfigWrapper> data = mappingConfigFacade.precDataMatch_remote(file, type);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "数据导入[by:zhaops]",
            notes = "file（必填）：导入文件 <br>" +
                    "hospitalId（必填）：医院id <br>" +
                    "uesrId（必填）：操作人id <br>" +
                    "type（必填）： 类型：1-化验、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候、14-麻醉 <br>")
    @PostMapping(value = "/importExcel", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @SysLogger("importExcel")
    @Transactional
    public RespDTO<Boolean> importExcel(@RequestParam("file") MultipartFile file,
                            @RequestParam("hospitalId") Long hospitalId,
                            @RequestParam("type") Integer type,
                            @RequestParam("uesrId") String userId) {
        Boolean data = mappingConfigFacade.importExcel(file, hospitalId, type, userId);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "数据导出[by:zhaops]",
            notes = "hospitalId（必填）：医院id <br>" +
                    "isMatch（选填，默认全部）：是否匹配（0-未匹配、1-已匹配、2-多条匹配） <br>" +
                    "type（必填）： 类型：1-化验、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候、14-麻醉 <br>")
    @PostMapping("/exportExcel")
    @SysLogger("exportExcel")
    public void exportExcel(HttpServletResponse response, @RequestBody @Valid MappingConfigPageVO mappingConfigPageVO) {
        mappingConfigFacade.exportExcel(response, mappingConfigPageVO);
    }

    @ApiOperation(value = "数据导出_远程调用[by:zhaops]",
            notes = "")
    @PostMapping("/exportExcel_remote")
    @SysLogger("exportExcel_remote")
    @ApiIgnore
    public RespDTO<List<MappingConfigWrapper>> exportExcel_remote(@RequestBody @Valid MappingConfigPageVO mappingConfigPageVO) {
        List<MappingConfigWrapper> data = mappingConfigFacade.exportExcel_remote(mappingConfigPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "分页查询[by:zhaops]",
            notes = "hospitalId（必填）：医院id <br>" +
                    "isMatch（选填，默认全部）：是否匹配（0-未匹配、1-已匹配、2-多条匹配） <br>" +
                    "hisName（必填）：医院术语名称 <br>" +
                    "uniqueName（选填，可为空）：标准术语名称 <br>" +
                    "type（必填）： 类型：1-化验、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候、14-麻醉 <br>")
    @PostMapping("/getPage")
    @SysLogger("getPage")
    public RespDTO<Page<MappingConfigWrapper>> getPage(@RequestBody @Valid MappingConfigPageVO mappingConfigPageVO) {
        IPage<MappingConfigWrapper> data = mappingConfigFacade.getPage(mappingConfigPageVO);
        Page<MappingConfigWrapper> page = new Page<>();
        BeanUtil.copyProperties(data, page);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "获取映射关系[by:zhaops]",
            notes = "id：关联关系id")
    @PostMapping("/getRecord")
    @SysLogger("getRecord")
    @Transactional
    public RespDTO<MappingConfigWrapper> getRecord(@RequestBody @Valid IdVO idVO) {
        MappingConfigWrapper data = mappingConfigFacade.getRecord(idVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "映射关系是否已存在[by:zhaops]",
            notes = "id：关联关系id <br>" +
                    "hospitalId（必填）：医院id <br>" +
                    "hisName（必填）：医院术语名称 <br>" +
                    "hisDetailName（选填，可为空）：医院术语名称-细项 <br>" +
                    "conceptId（必填）：标准术语id <br>" +
                    "formConceptId（选填，可为空）：药品剂型id，选填 <br>" +
                    "source（必填）：数据来源：1-标准词、2-同义词、3-编码、4-历史数据、5-相似词 <br>" +
                    "type（必填）： 类型：1-化验、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候、14-麻醉 <br>")
    @PostMapping("/isExistRecord")
    @SysLogger("isExistRecord")
    public RespDTO<Boolean> isExistRecord(@RequestBody @Valid MappingConfig mappingConfig) {
        Boolean data = mappingConfigFacade.isExistRecord(mappingConfig);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "查询已映射关系[by:zhaops]",
            notes = "hospitalId（必填）：医院id <br>" +
                    "hisName（必填）：医院术语名称 <br>" +
                    "hisDetailName（选填，可为空）：医院术语名称-细项 <br>" +
                    "type（必填）： 类型：1-化验、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候、14-麻醉 <br>")
    @PostMapping("/getRelatedMapping")
    @SysLogger("getRelatedMapping")
    public RespDTO<List<MappingConfigWrapper>> getRelatedMapping(@RequestBody @Valid MappingConfigVO mappingConfigVO) {
        List<MappingConfigWrapper> data = mappingConfigFacade.getRelatedMapping(mappingConfigVO);
        return RespDTO.onSuc(data);
    }


    @ApiOperation(value = "保存或修改映射关系[by:zhaops]",
            notes = "id：关联关系id <br>" +
                    "hospitalId（必填）：医院id <br>" +
                    "hisName（必填）：医院术语名称 <br>" +
                    "hisDetailName（选填，可为空）：医院术语名称-细项 <br>" +
                    "conceptId（必填）：标准术语id <br>" +
                    "formConceptId（选填，可为空）：药品剂型id，选填 <br>" +
                    "source：数据来源（必填）：1-标准词、2-同义词、3-编码、4-历史数据、5-相似词 <br>" +
                    "type（必填）： 类型：1-化验、3-辅检、4-诊断、5-药品、6-手术和操作、7-科室、8-输血、10-量表、11-护理、12-中医诊断、13-中医证候、14-麻醉 <br>")
    @PostMapping("/saveOrUpdateRecord")
    @SysLogger("saveOrUpdateRecord")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecord(@RequestBody @Valid MappingConfig mappingConfig) {
        Boolean data = mappingConfigFacade.saveOrUpdateRecord(mappingConfig);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "删除映射关系[by:zhaops]",
            notes = "id：关联关系id")
    @PostMapping("/deleteRecord")
    @SysLogger("deleteRecord")
    @Transactional
    public RespDTO<Boolean> deleteRecord(@RequestBody @Valid IdVO idVO) {
        Boolean data = mappingConfigFacade.deleteRecord(idVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "批量删除映射关系[by:zhaops]",
            notes = "ids：关联关系id列表")
    @PostMapping("/deleteRecords")
    @SysLogger("deleteRecords")
    @Transactional
    public RespDTO<Boolean> deleteRecords(@RequestBody @Valid IdListVO idListVO) {
        Boolean data = mappingConfigFacade.deleteRecords(idListVO);
        return RespDTO.onSuc(data);
    }
}